// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>

static double n_count=0.0;
static double p_count=0.0;
static double rn_count=0.0;
static double rp_count=0.0;


double carrier_count_get_n(struct device *in)
{
return n_count;
}

double carrier_count_get_p(struct device *in)
{
return p_count;
}

double  carrier_count_get_rn(struct device *in)
{
return rn_count;
}

double carrier_count_get_rp(struct device *in)
{
return rp_count;
}

void carrier_count_reset(struct device *in)
{
n_count=0.0;
p_count=0.0;
rn_count=0.0;
rp_count=0.0;
}

double get_J_recom(struct device *in)
{
int i=0;
double Rtot=0.0;
double dx=in->ymesh[1]-in->ymesh[0];
for (i=0;i<in->ymeshpoints;i++)
{
Rtot+=(in->Rbi[i]-in->Gex[i]*in->Dphotoneff)*dx;
//printf("%e %e\n",in->Rbi[i],in->Gex[i]*in->Dphotoneff);
}
//getchar();
return Rtot*q;
}

void carrier_count_add(struct device *in)
{
int i;
double locat_n_tot=0.0;
double locat_p_tot=0.0;

n_count+=(in->Jn[0][0]+in->Jn[in->ymeshpoints-1][0])*in->dt/q;
p_count+=(in->Jp[0][0]+in->Jp[in->ymeshpoints-1][0])*in->dt/q;


double dx=in->ymesh[1]-in->ymesh[0];
//printf("\n\n%e %e\n\n",dx*(double)in->ymeshpoints,in->ymesh[in->ymeshpoints-1]);
for (i=0;i<in->ymeshpoints;i++)
{
locat_n_tot+=in->Rbi[i]*dx;
locat_p_tot+=in->Rbi[i]*dx;
}
//printf("%e\n",in->ymesh[in->ymeshpoints-1]);
//getchar();
rn_count+=locat_n_tot*in->dt;
rp_count+=locat_p_tot*in->dt;

//printf("\n%e %e\n",rn_count,rp_count);
}

double get_extracted_np(struct device *in)
{
int i;
int band;
int bands=in->bands;
double sum_n=0.0;
double sum_p=0.0;
for (i=0;i<in->ymeshpoints;i++)
{
	for (band=0;band<bands;band++)
	{
		sum_n+=(in->n[i][band]-in->ninit[i]);
		sum_p+=(in->p[i][band]-in->pinit[i]);

		//if (temp>0.0)
		//printf("%e %e\n",(in->n[i][band]-in->ninit[i]),in->p[i][band]-in->pinit[i]);
		//getchar();
	}
}


return (sum_n+sum_p)/((double)(in->ymeshpoints))/2.0;
}

double get_extracted_n(struct device *in)
{
int i;
int band;
int bands=in->bands;

double tot=0.0;
for (i=0;i<in->ymeshpoints;i++)
{
	for (band=0;band<bands;band++)
	{
		tot+=(in->n[i][band]-in->ninit[i]);
		//printf("%e %e\n",(in->n[i][band]-in->ninit[i]),in->p[i][band]-in->pinit[i]);
		//getchar();
	}
}


return tot/((double)(in->ymeshpoints));
}

double get_extracted_p(struct device *in)
{
int i;
int band;
int bands=in->bands;

double tot=0.0;
for (i=0;i<in->ymeshpoints;i++)
{
	for (band=0;band<bands;band++)
	{
		tot+=(in->p[i][band]-in->pinit[i]);
		//printf("%e %e\n",(in->n[i][band]-in->ninit[i]),in->p[i][band]-in->pinit[i]);
		//getchar();
	}
}


return tot/((double)(in->ymeshpoints));
}

double get_extracted_k(struct device *in)
{
int i;

double tot=0.0;
double n=0.0;
for (i=0;i<in->ymeshpoints;i++)
{
	tot+=in->Rbi[i];
	n+=in->n[i][0]*in->p[i][0];
}
//printf("%le %le %le\n",tot,n,tot/n);
//getchar();
return tot/n;
}

double get_avg_mue(struct device *in)
{
int i;
int band;
int bands=in->bands;

double tot=0.0;
for (i=0;i<in->ymeshpoints;i++)
{
	for (band=0;band<bands;band++)
	{
		tot+=in->mun[i][band];
	}
}
return tot/(((double)in->ymeshpoints*bands));
}

double get_avg_muh(struct device *in)
{
int i;
int band;
int bands=in->bands;

double tot=0.0;
for (i=0;i<in->ymeshpoints;i++)
{
	for (band=0;band<bands;band++)
	{
		tot+=in->mup[i][band];
	}
}
return tot/(((double)in->ymeshpoints*bands));
}


double get_total_n_charge(struct device *in)
{
int i;
int band;
int bands=in->bands;

double tot=0.0;
for (i=0;i<in->ymeshpoints;i++)
{
	for (band=0;band<bands;band++)
	{
		tot+=in->n[i][band];
	}
}

return tot;
}

double get_total_p_charge(struct device *in)
{
int i;
int band;
int bands=in->bands;

double tot=0.0;
for (i=0;i<in->ymeshpoints;i++)
{
	for (band=0;band<bands;band++)
	{
		tot+=in->p[i][band];
	}
}

return tot;
}

double get_total_n_trapped_charge(struct device *in)
{
int i;

double tot=0.0;
for (i=0;i<in->ymeshpoints;i++)
{
		tot+=in->nt[i];
}

return tot;
}

double get_total_p_trapped_charge(struct device *in)
{
int i;

double tot=0.0;
for (i=0;i<in->ymeshpoints;i++)
{
	tot+=in->pt[i];
}

return tot;
}


double get_I(struct device *in)
{
int band;
int bands=in->bands;
double ret=0.0;
for (band=0;band<bands;band++)
{
ret+=(in->Jn[0][band]+in->Jp[0][band]+in->Jn[in->ymeshpoints-1][band]+in->Jp[in->ymeshpoints-1][band])*(in->xlen*in->zlen)/2.0;
}
return ret;
}

double get_equiv_J(struct device *in)
{
double J=0.0;
J=get_J_recom(in);

J+=in->Vapplied/in->Rshunt/(in->xlen*in->zlen);
//printf("%e %e\n",(in->xlen*in->zlen)*100*100,in->Rshunt);
return J;
}

double get_equiv_V(struct device *in)
{
double J=get_J_recom(in);
double V=J*in->Rcontact*in->xlen*in->zlen+in->Vapplied;
//printf("Rcontact = %e %e\n",in->Rcontact,in->xlen*in->zlen);
return V;
}

double mod(double x)
{
if (x<0) return -x;
return x;
}

double get_J(struct device *in)
{
//int i;
double ret=0.0;
//int band;
//int bands=in->bands;

//for (i=0;i<in->ymeshpoints;i++)
//{
	//for (band=0;band<bands;band++)
	//{
	ret=(get_J_left(in)+get_J_right(in))/2.0;
	//}
//}

return ret;
}

double get_J_left(struct device *in)
{

double ret=0.0;
ret=in->Jpleft-in->Jnleft;

return ret*q;
}

double get_J_right(struct device *in)
{

double ret=0.0;
ret=in->Jnright-in->Jpright;
//printf("%e %e\n",in->Jn[0][0],in->Jn[0][in->ymeshpoints-2]);
//getchar();
return ret*q;
}

