// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#define _FILE_OFFSET_BITS 64
#define _LARGEFILE_SOURCE
//<clean=all></clean>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

#include "sim.h"

/**Rescale the scale and the data
@param in The structure holding the data
@param xmul multiply x axis by this
@param ymul multiply y axis by this
*/
void inter_rescale(struct istruct* in,double xmul, double ymul)
{
int i;
for  (i=0;i<in->len;i++)
{
	in->x[i]*=xmul;
	in->data[i]*=ymul;
}

}

/**Make all the data positive
@param in the structure holding the data
*/
void inter_mod(struct istruct* in)
{
int i;
for  (i=0;i<in->len;i++)
{
if (in->data[i]<0.0) in->data[i]*=-1.0;
}

}

void inter_norm(struct istruct* in,double mul)
{
int i;
double tot=0.0;
double dx=0.0;
for  (i=0;i<in->len;i++)
{
	if (i==0)
	{
		dx=in->x[1]-in->x[0];
	}else
	if (i==in->len-1)
	{
		dx=in->x[i]-in->x[in->len-2];
	}else
	{
		dx=(in->x[i+1]-in->x[i])/2.0+(in->x[i]-in->x[i-1])/2.0;
	}

	tot+=dx*in->data[i];
}

for  (i=0;i<in->len;i++)
{

	in->data[i]/=tot;
	in->data[i]*=mul;

}

}


/**Load data from a file
@param in the structure holding the data
@param name The file name.
*/
void inter_load(struct istruct* in,char *name,int len,int col,int ncol)
{
FILE *file;
file=fopen(name,"r");
in->len=len;
int i=0;
double out=0.0;
in->x = (double *)malloc (in->len*sizeof(double));
in->data = (double *)malloc (in->len*sizeof(double));
int ii=0;
for  (i=0;i<in->len;i++)
{
	fscanf(file,"%lf",&(in->x[i]));
	for (ii=0;ii<ncol;ii++)
	{
		fscanf(file,"%lf",&(out));
		if (ii==col) in->data[i]=out;
	}

}

fclose(file);
}

void inter_save(struct istruct* in,char *name)
{
FILE *file;
file=fopen(name,"w");
int i=0;
for  (i=0;i<in->len;i++)
{
fprintf(file,"%e %e\n",in->x[i],in->data[i]);
}

fclose(file);
}

/**Get interpolated data from a data set
@param in The structure holding the data
@param x the position of the data.
@return the interpolated data value
*/
double inter_get(struct istruct* in,double x)
{
double x0;
double x1;
double y0;
double y1;

double ret;
int i=0;

//if (x>in->x[in->len-1]) return 0.0;
if (x<in->x[0]) return 0.0;



for (i=0;i<in->len;i++)
{
if (in->x[i]>x) break;
}

i--;
if (i<0) i=0;

if (i>=(in->len-1)) i=in->len-2;

x0=in->x[i];
x1=in->x[i+1];

y0=in->data[i];
y1=in->data[i+1];

ret=y0+((y1-y0)/(x1-x0))*(x-x0);
return ret;
}


/**Print the interpolated data
@param in The structure holding the data
*/
void inter_print(struct istruct* in)
{

int i;

for  (i=0;i<in->len;i++)
{
	printf("%lf %e\n",in->x[i],in->data[i]);
}

}

/**Free the structure holding the data
@param in The structure holding the data
*/
void inter_free(struct istruct* in)
{
in->len=0;
free(in->x);
free(in->data);
}
