// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>

static double *Jnl=NULL;
static double *Jnr=NULL;
static double *Jpl=NULL;
static double *Jpr=NULL;

static double *Dnl=NULL;
static double *Dnc=NULL;
static double *Dnr=NULL;
static double *Dpl=NULL;
static double *Dpc=NULL;
static double *Dpr=NULL;

static double *nl=NULL;
static double *nc=NULL;
static double *nr=NULL;

static double *pl=NULL;
static double *pc=NULL;
static double *pr=NULL;

static double *xil=NULL;
static double *xir=NULL;
static double *xipl=NULL;
static double *xipr=NULL;

static double *dJpdxipl=NULL;
static double *dJpdxipc=NULL;
static double *dJpdxipr=NULL;

static double *dnl=NULL;
static double *dnc=NULL;
static double *dnr=NULL;

static double *dpl=NULL;
static double *dpc=NULL;
static double *dpr=NULL;

static double *munl=NULL;
static double *munc=NULL;
static double *munr=NULL;

static double *mupl=NULL;
static double *mupc=NULL;
static double *mupr=NULL;


static double *wnl=NULL;
static double *wnc=NULL;
static double *wnr=NULL;

static double *wpl=NULL;
static double *wpc=NULL;
static double *wpr=NULL;

static double *dnldTe=NULL;
static double *dncdTe=NULL;
static double *dnrdTe=NULL;

static double *dpldTh=NULL;
static double *dpcdTh=NULL;
static double *dprdTh=NULL;

static double *nc_all=NULL;
static double *pc_all=NULL;
static double *dnc_all=NULL;
static double *dpc_all=NULL;


static double *dJdxil=NULL;
static double *dJdxic=NULL;
static double *dJdxir=NULL;

static double *dJdphil=NULL;
static double *dJdphic=NULL;
static double *dJdphir=NULL;

static double *dJpdphil=NULL;
static double *dJpdphic=NULL;
static double *dJpdphir=NULL;


static double *dphidxic=NULL;
static double *dphidxipc=NULL;


void quick_update();

#ifdef only
int only_update_thermal=FALSE;
#endif

void update_solver_vars(struct device *in,int clamp)
{
int i;
int band;
int bands=in->bands;
double update=0.0;
	for (i=0;i<in->ymeshpoints;i++)
	{
#ifdef only
		if (only_update_thermal==FALSE)
		{
#endif
			update=in->b[i];
			if (clamp==TRUE)
			{
				in->phi[i]+=update/(1.0+fabs(update/in->electrical_clamp/(300.0*kb/q)));
			}else
			{
				in->phi[i]+=update;

			}

			for (band=0;band<bands;band++)
			{
				update=(in->b[in->ymeshpoints*(1+band)+i]);
				if (clamp==TRUE)
				{
					in->x[i][band]+=update/(1.0+fabs(update/in->electrical_clamp/(300.0*kb/q)));
				}else
				{
					in->x[i][band]+=update;
				}
			}


			for (band=0;band<bands;band++)
			{
				update=(in->b[in->ymeshpoints*(1+bands+band)+i]);
				if (clamp==TRUE)
				{
					in->xp[i][band]+=update/(1.0+fabs(update/in->electrical_clamp/(300.0*kb/q)));//
				}else
				{
					in->xp[i][band]+=update;

				}
			}

			/*
			update=(in->b[i+3*in->ymeshpoints]);
			if (clamp==TRUE)
			{
				in->ex[i]+=update/(1.0+fabs(update/1e30/(300.0*kb/q)));;
			}else
			{
				in->ex[i]+=update;

			}


		if (in->nnewton==TRUE)
		{
			update=(in->b[in->ymeshpoints*(1+bands+bands)+i]);
			if (clamp==TRUE)
			{
				in->Te[i]+=update/(1.0+fabs(update/in->thermal_clamp/(300.0*kb/q)));
			}else
			{
				in->Te[i]+=update;

			}
		}
*/

#ifdef only

		}else
		{
#endif

		if (in->ntrapnewton==TRUE)
		{
			update=(in->b[in->ymeshpoints*(1+bands+bands)+i]);
			if (clamp==TRUE)
			{
				in->xt[i]+=update/(1.0+fabs(update/in->electrical_clamp/(300.0*kb/q)));
			}else
			{
				in->xt[i]+=update;

			}
		}

		if (in->ptrapnewton==TRUE)
		{
			update=(in->b[in->ymeshpoints*(1+bands+bands+1)+i]);
			if (clamp==TRUE)
			{
				in->xpt[i]+=update/(1.0+fabs(update/in->electrical_clamp/(300.0*kb/q)));
			}else
			{
				in->xpt[i]+=update;

			}
		}


		if (in->hnewton==TRUE)
		{
			update=(in->b[i+5*in->ymeshpoints]);
			if (clamp==TRUE)
			{
				in->Th[i]+=update/(1.0+fabs(update/in->thermal_clamp/(300.0*kb/q)));
			}else
			{
				in->Th[i]+=update;

			}
		}

		if (in->lnewton==TRUE)
		{
			update=(in->b[i+6*in->ymeshpoints]);
			if (clamp==TRUE)
			{
				in->Tl[i]+=update/(1.0+fabs(update/in->thermal_clamp/(300.0*kb/q)));
			}else
			{
				in->Tl[i]+=update;

			}
		}


#ifdef only

		}
#endif

#ifdef only
	if (only_update_thermal==FALSE)
	{
#endif
		for (band=0;band<in->bands;band++)
		{
			in->Fn[i][band]=in->x[i][band]-in->phi[i];
			in->Fp[i][band]=-in->xp[i][band]-in->phi[i];
		}

		in->Fnt[i]=in->xt[i]-in->phi[i];
		in->Fpt[i]=-in->xpt[i]-in->phi[i];
		//printf("%e\n",in->Fpt[i]);

		in->Ec[i]=-in->phi[i]-in->Xi[i];
		in->Ev[i]=-in->phi[i]-in->Xi[i]-in->Eg[i];


		in->mun_avg[i]=0.0;
		in->mup_avg[i]=0.0;

		for (band=0;band<in->bands;band++)
		{
			in->dn[i][band]=get_dn_den(in->x[i][band]+in->t[i][band],in->Te[i],in->me[i],in->dostype[i],dos_all);
			in->n[i][band]=get_n_den(in->x[i][band]+in->t[i][band],in->Te[i],in->me[i],in->dostype[i],dos_all);
			in->dndTe[i][band]=get_dndT_den(in->x[i][band]+in->t[i][band],in->Te[i],in->me[i],in->dostype[i],dos_all);
			in->dp[i][band]=get_dp_den(in->xp[i][band]-in->tp[i][band],in->Th[i],in->mh[i],in->dostype[i],dos_all);
			in->p[i][band]=get_p_den(in->xp[i][band]-in->tp[i][band],in->Th[i],in->mh[i],in->dostype[i],dos_all);
			//if (in->Vapplied<0.1)
			//{
			in->mun[i][band]=get_n_mu(in->x[i][band]+in->t[i][band],in->Te[i],in->me[i],in->dostype[i]);
			//printf("%e\n",in->mun[i][band]);
			in->mup[i][band]=get_p_mu(in->xp[i][band]-in->tp[i][band],in->Th[i],in->mh[i],in->dostype[i]);//get_mat_param_mobility(&(in->mat.l[in->ymatlayers[i]]),mat_muh,band);
			//if (i==0) in->mun[i][band]=1e-14;
			//if (i==in->ymeshpoints-1) in->mup[in->ymeshpoints-1][band]=1e-14;

			//printf("update\n");
			//}
			//printf("%e %e\n",get_mat_param_mobility(&(in->mat.l[in->ymatlayers[i]]),mat_mue,band),get_n_mu(in->x[i][band]+in->t[i][band],in->Te[i],in->me[i],in->dostype[i]));

			in->mun_avg[i]+=in->mun[i][band];
			in->mup_avg[i]+=in->mup[i][band];

		}

		in->mun_avg[i]/=(double)bands;
		in->mup_avg[i]/=(double)bands;

//printf("%e %e\n",in->mun_avg[i],in->mup_avg[i]);

		if (in->ntrapnewton)
		{
			in->nt[i]=get_n_trap_den(in->xt[i]+in->tt[i],in->Te[i],in->me[i],in->dostype[i],0);
			in->dnt[i]=get_dn_trap_den(in->xt[i]+in->tt[i],in->Te[i],in->me[i],in->dostype[i],0);
		}else if (in->combitrap)
		{
			in->nt[i]=get_n_trap_den(in->x[i][0]+in->t[i][0],in->Te[i],in->me[i],in->dostype[i],0);
			in->dnt[i]=get_dn_trap_den(in->x[i][0]+in->t[i][0],in->Te[i],in->me[i],in->dostype[i],0);
		}else
		{
			in->nt[i]=0.0;
			in->dnt[i]=0.0;
		}


		if (in->ptrapnewton)
		{
			in->pt[i]=get_p_trap_den(in->xpt[i]-in->tpt[i],in->Th[i],in->mh[i],in->dostype[i],0);
			in->dpt[i]=get_dp_trap_den(in->xpt[i]-in->tpt[i],in->Th[i],in->mh[i],in->dostype[i],0);
		}else if (in->combitrap)
		{
			in->pt[i]=get_p_trap_den(in->xp[i][0]-in->tp[i][0],in->Th[i],in->mh[i],in->dostype[i],0);
			in->dpt[i]=get_dp_trap_den(in->xp[i][0]-in->tp[i][0],in->Th[i],in->mh[i],in->dostype[i],0);
		}
		else
		{
			in->dpt[i]=0.0;
			in->pt[i]=0.0;
		}


		in->ni[i]=sqrt(get_n_den(in->Fi[i]-in->Ec[i],in->Te[i],in->me[i],in->dostype[i],dos_all)*get_p_den(in->Ev[i]-in->Fi[i],in->Th[i],in->mh[i],in->dostype[i],dos_all));
//printf("%e\n",sqrt(get_n_den(in->Fi[i]-in->Ec[i],in->Te[i],in->me[i],in->dostype[i],dos_all,0.0)*get_p_den(in->Ev[i]-in->Fi[i],in->Th[i],in->mh[i],in->dostype[i],dos_all)));
		}
		//getchar();

//printf("asad");
//getchar();

#ifdef only
	}
#endif

}

void fill_matrix(struct device *in)
{
int band;
int bands=in->bands;
//FILE *file_j =fopen("myj.dat","w");
//getchar();
double phil;
double phic;
double phir;
double yl;
double yc;
double yr;
double dyl;
double dyr;
double ddh;
double dh;
int pos=0;

double Ecl=0.0;
double Ecr=0.0;
double Ecc=0.0;
double Evl=0.0;
double Evr=0.0;
double Evc=0.0;

double Tll=300.0;
double Tlc=300.0;
double Tlr=300.0;

double Tel=300.0;
double Tec=300.0;
double Ter=300.0;

double Thl=300.0;
double Thc=300.0;
double Thr=300.0;

double ni=0.0;



double xnr;
double tnr;
double xnl;
double tnl;

double xpr;
double tpr;
double xpl;
double tpl;


double exl;
double exr;
double exc;
double Dexl;
double Dexc;
double Dexr;
double R;

double epr;
double epc;
double epl;

//double dexl;
//double dexc;
//double dexr;
//double dex;
double G;
double Gex;
int i;
double dJdxipc=0.0;
double dJpdxic=0.0;

double mel=0.0;
double mec=0.0;
double mer=0.0;

double mhl=0.0;
double mhc=0.0;
double mhr=0.0;


double e0=0.0;
double e1=0.0;

double dphil=0.0;
double dphic=0.0;
double dphir=0.0;
double Rbi=0.0;
double Rbi_mul=1.0;
double deriv=0.0;

double nlast=0.0;
double plast=0.0;
double dt=0.0;

//double dTel;
//double dTec;
//double dTer;
//double dTe;


//double dJnldTe=0.0;
//double dJncdTe=0.0;
//double dJnrdTe=0.0;
//double dphidTe=0.0;
//double dTeldn=0.0;
//double dTecdn=0.0;
//double dTerdn=0.0;
//double dThl=0.0;
//double dThc=0.0;
//double dThr=0.0;
//double dTh=0.0;

//double dThldp=0.0;
//double dThcdp=0.0;
//double dThrdp=0.0;

double kll=0.0;
double klc=0.0;
double klr=0.0;

double kl0=0.0;
double kl1=0.0;

//double dTll=0.0;
//double dTlc=0.0;
//double dTlr=0.0;
//double dTl=0.0;

//double dTlcdTe=0.0;
//double dTlcdTh=0.0;

//double dTlcdn=0.0;
//double dTlcdp=0.0;

//double dJpldTh=0.0;
//double dJpcdTh=0.0;
//double dJprdTh=0.0;

//double dSnl_predTe=0.0;
//double dSnr_predTe=0.0;

//double dTeldphi=0.0;
//double dTecdphi=0.0;
//double dTerdphi=0.0;

//double Snl=0.0;
//double Snr=0.0;
//double Snl_pre=0.0;
//double Snr_pre=0.0;


//double dsxilpdThl=0.0;
//double dsxilpdThc=0.0;
//double dsxirpdThc=0.0;
//double dsxirpdThr=0.0;

//double sxipl=0.0;
//double sxipr=0.0;

//double Spl=0.0;
//double Spr=0.0;

//double Spl_pre=0.0;
//double Spr_pre=0.0;
//double dSpl_predTh=0.0;
//double dSpr_predTh=0.0;

//double dxildTel=0.0;
//double dxildTec=0.0;
//double dxirdTec=0.0;
//double dxirdTer=0.0;

//double dThldphi=0.0;
//double dThcdphi=0.0;
//double dThrdphi=0.0;

//double dxilpdThl=0.0;
//double dxilpdThc=0.0;

//double dxirpdThc=0.0;
//double dxirpdThr=0.0;


double tauncapc=0.0;
double taunescapec=0.0;
double ntc=0.0;



double taupcapc=0.0;
double taupescapec=0.0;
double ptc=0.0;


int forcecharge=FALSE;
double one;


double dntrapdn=0.0;
double dntrapdntrap=0.0;
double dphidntrap=0.0;
double dntc=0.0;
double dntrap=0.0;

double dptrapdp=0.0;
double dptrapdptrap=0.0;
double dphidptrap=0.0;
double dptc=0.0;
double dptrap=0.0;

double length=in->ymesh[in->ymeshpoints-1];
double n_sum=0.0;
double p_sum=0.0;
double np_sum=0.0;
double l=0.0;
double r=0.0;
double delta=0.0;
double deibel=0.0;
if (in->Rbitype==5)
{
	for (i=0;i<in->ymeshpoints;i++)
	{
		l=in->ymesh[i];
		if (i==in->ymeshpoints-1)
		{
			r=in->ymesh[i]+in->ymesh[i]-in->ymesh[i-1];
		}else
		{
			r=in->ymesh[i+1];
		}

		delta=r-l;

		n_sum+=in->n[i][0]*delta;
		p_sum+=in->p[i][0]*delta;
		np_sum+=in->n[i][0]*in->p[i][0]*delta;

	}
n_sum/=length;
p_sum/=length;
np_sum/=length;
deibel=np_sum/(n_sum*p_sum);
}

//calculate_kd(in);


		pos=0;
		for (i=0;i<in->ymeshpoints;i++)
		{

			if (i==0)
			{
				exl=0.0;
				Dexl=in->Dex[0];
				phil=in->Vapplied;
				yl=in->ymesh[0]-(in->ymesh[1]-in->ymesh[0]);
				mel=in->me[0];
				mhl=in->mh[0];
				Tll=in->Tll;
				Tel=in->Tll;
				Thl=in->Tll;




				Ecl=-in->Xi[0]-phil;
				Evl=-in->Xi[0]-phil-in->Eg[0];
				epl=in->epsilonr[0]*epsilon0;

				for (band=0;band<bands;band++)
				{
					xnl=in->Fi[0];
					tnl=in->Xi[0];
					if (forcecharge==TRUE)
					{
						one=in->deltaFln;
					}else
					{
						one=xnl+tnl;
						//one/=2.0;
						//printf("%e\n",one);
						//getchar();
					}

					nl[band]=get_n_den(one,Tel,mel,in->bdos,dos_all);

					dnl[band]=get_dn_den(one,Tel,mel,in->bdos,dos_all);
					wnl[band]=get_n_w(one,Tel,mel,in->bdos);
					dnldTe[band]=0.0;
					munl[band]=in->mun[0][band];
				}


				for (band=0;band<bands;band++)
				{
					xpl=-in->Fi[0];
					tpl=in->Xi[0]+in->Eg[0];

					if (forcecharge==TRUE)
					{
						one=in->deltaFlp;
					}else
					{
						one=xpl-tpl;
					}

					pl[band]=get_p_den(one,Thl,mhl,in->bdos,dos_all);

					dpl[band]=get_dp_den(one,Thl,mhl,in->bdos,dos_all);
					wpl[band]=get_p_w(one,Thl,mhl,in->bdos);
					dpldTh[band]=0.0;
					mupl[band]=in->mup[0][band];
				}
					//printf("left n= %e p= %e \n",nl[0],pl[0]);


				kll=in->kl[i];

			}else
			{
				Dexl=in->Dex[i-1];
				exl=in->ex[i-1];
				phil=in->phi[i-1];
				yl=in->ymesh[i-1];
				Tll=in->Tl[i-1];
				Tel=in->Te[i-1];
				Thl=in->Th[i-1];
				Ecl=in->Ec[i-1];
				Evl=in->Ev[i-1];

				for (band=0;band<bands;band++)
				{
					mel=in->me[i-1];
					mhl=in->mh[i-1];

					nl[band]=in->n[i-1][band];
					dnl[band]=in->dn[i-1][band];

					dnldTe[band]=get_dndT_den(in->x[i-1][band]+in->t[i-1][band],Tel,mel,in->dostype[i],dos_all);
					wnl[band]=get_n_w(in->x[i-1][band]+in->t[i-1][band],Tel,mel,in->dostype[i]);

					dpldTh[band]=get_dpdT_den(in->xp[i-1][band]-in->tp[i-1][band],Thl,mhl,in->dostype[i],dos_all);
					wpl[band]=get_p_w(in->xp[i-1][band]-in->tp[i-1][band],Thl,mhl,in->dostype[i]);

					pl[band]=in->p[i-1][band];
					dpl[band]=in->dp[i-1][band];
					munl[band]=in->mun[i-1][band];
					mupl[band]=in->mup[i-1][band];
				}

				epl=in->epsilonr[i-1]*epsilon0;


				kll=in->kl[i-1];
			}

			Ecc=-in->Xi[i]-in->phi[i];
			Evc=-in->Xi[i]-in->phi[i]-in->Eg[i];

			if (i==(in->ymeshpoints-1))
			{

				Dexr=in->Dex[i];
				exr=0.0;
				phir=in->Vr;
				yr=in->ymesh[i]+(in->ymesh[i]-in->ymesh[i-1]);
				Tlr=in->Tlr;
				Ter=in->Tlr;
				Thr=in->Tlr;

				Ecr=-in->Xi[i]-phir;
				Evr=-in->Xi[i]-phir-in->Eg[i];

				mer=in->me[i];
				mhr=in->mh[i];

				for (band=0;band<bands;band++)
				{
					xnr=phir+in->Fi[i];
					tnr=in->Xi[i];
					if (forcecharge==TRUE)
					{
						one=in->deltaFrn;
					}else
					{
						one=xnr+tnr;
					}
					nr[band]=get_n_den(one,Ter,mer,in->bdos,dos_all);
					dnr[band]=get_dn_den(one,Ter,mer,in->bdos,dos_all);
					//printf("rod = %e\n",nr);
					wnr[band]=get_n_w(one,Ter,mer,in->bdos);
					dnrdTe[band]=0.0;


					xpr=-(phir+in->Fi[i]);
					tpr=in->Xi[i]+in->Eg[i];
					if (forcecharge==TRUE)
					{
						one=in->deltaFrp;
					}else
					{
						one=xpr-tpr;
						//one/=2.0;
					}
					pr[band]=get_p_den(one,Thr,mhr,in->bdos,dos_all);
					dpr[band]=get_dp_den(one,Thr,mhr,in->bdos,dos_all);
					wpr[band]=get_p_w(one,Thr,mhr,in->bdos);
					dprdTh[band]=0.0;
					munr[band]=in->mun[i][band];
					mupr[band]=in->mup[i][band];
				}

				//printf("right n= %e p= %e \n",nr[0],pr[0]);

				epr=in->epsilonr[i]*epsilon0;
				klr=in->kl[i];

			}else
			{
				mer=in->me[i+1];
				mhr=in->mh[i+1];

				Dexr=in->Dex[i+1];
				exr=in->ex[i+1];
				phir=in->phi[i+1];
				yr=in->ymesh[i+1];
				Tlr=in->Tl[i+1];
				Ter=in->Te[i+1];
				Thr=in->Th[i+1];

				Ecr=in->Ec[i+1];
				Evr=in->Ev[i+1];

				for (band=0;band<bands;band++)
				{
					nr[band]=in->n[i+1][band];
					dnr[band]=in->dn[i+1][band];
					dnrdTe[band]=get_dndT_den(in->x[i+1][band]+in->t[i+1][band],in->Te[i+1],mer,in->dostype[i],dos_all);
					wnr[band]=get_n_w(in->x[i+1][band]+in->t[i+1][band],in->Te[i+1],mer,in->dostype[i]);

					dprdTh[band]=get_dpdT_den(in->xp[i+1][band]-in->tp[i+1][band],in->Th[i+1],mhr,in->dostype[i],dos_all);
					wpr[band]=get_p_w(in->xp[i+1][band]-in->tp[i+1][band],in->Th[i+1],mhr,in->dostype[i]);

					pr[band]=in->p[i+1][band];
					dpr[band]=in->dp[i+1][band];
					munr[band]=in->mun[i+1][band];
					mupr[band]=in->mup[i+1][band];
				}
				epr=in->epsilonr[i+1]*epsilon0;
				klr=in->kl[i+1];

			}


			tauncapc=in->tauncap[i];
			taunescapec=in->taunescape[i];

			taupcapc=in->taupcap[i];
			taupescapec=in->taupescape[i];

			mec=in->me[i];
			mhc=in->mh[i];

			epc=in->epsilonr[i]*epsilon0;


			exc=in->ex[i];
			Dexc=in->Dex[i];
			yc=in->ymesh[i];
			dyl=yc-yl;
			dyr=yr-yc;
			ddh=(dyl+dyr)/2.0;
			dh=(dyl+dyr);
			phic=in->phi[i];
			Tlc=in->Tl[i];
			Tec=in->Te[i];
			Thc=in->Th[i];
			ni=in->ni[i];

			for (band=0;band<bands;band++)
			{

				munc[band]=in->mun[i][band];
				mupc[band]=in->mup[i][band];


				//mupl[band]=(mupl[band]+mupr[band])/2.0;
				//mupc[band]=mupl[band];
				//mupr[band]=mupl[band];//mupl[band];

				//munl[band]=1e-7;//(munl[band]+munr[band])/2.0;
				//munc[band]=1e-7;
				//munr[band]=1e-7;//munl[band];

				dncdTe[band]=get_dndT_den(in->x[i][band]+in->t[i][band],in->Te[i],mec,in->dostype[i],dos_all);
				dpcdTh[band]=get_dpdT_den(in->xp[i][band]-in->tp[i][band],in->Th[i],mhc,in->dostype[i],dos_all);
				wnc[band]=get_n_w(in->x[i][band]+in->t[i][band],in->Te[i],mec,in->dostype[i]);
				wpc[band]=get_p_w(in->xp[i][band]-in->tp[i][band],in->Th[i],mhc,in->dostype[i]);

				Dnl[band]=munl[band]*(2.0/3.0)*wnl[band]/q;
				Dpl[band]=mupl[band]*(2.0/3.0)*wpl[band]/q;

				Dnc[band]=munc[band]*(2.0/3.0)*wnc[band]/q;
				Dpc[band]=mupc[band]*(2.0/3.0)*wpc[band]/q;

				Dnr[band]=munr[band]*(2.0/3.0)*wnr[band]/q;
				Dpr[band]=mupr[band]*(2.0/3.0)*wpr[band]/q;


				Dnl[band]=(Dnl[band]+Dnc[band])/2.0;
				Dnr[band]=(Dnr[band]+Dnc[band])/2.0;

				Dpl[band]=(Dpl[band]+Dpc[band])/2.0;
				Dpr[band]=(Dpr[band]+Dpc[band])/2.0;

				munl[band]=(munl[band]+munc[band])/2.0;
				munr[band]=(munr[band]+munc[band])/2.0;

				mupl[band]=(mupl[band]+mupc[band])/2.0;
				mupr[band]=(mupr[band]+mupc[band])/2.0;



				nc[band]=in->n[i][band];
				pc[band]=in->p[i][band];

				dnc[band]=in->dn[i][band];
				dpc[band]=in->dp[i][band];

				nc_all[band]=get_n_den(in->x[i][band]+in->t[i][band],in->Te[i],in->me[i],in->dostype[i],dos_all);
				pc_all[band]=get_p_den(in->xp[i][band]-in->tp[i][band],in->Th[i],in->mh[i],in->dostype[i],dos_all);
				dnc_all[band]=get_dn_den(in->x[i][band]+in->t[i][band],in->Te[i],in->me[i],in->dostype[i],dos_all);
				dpc_all[band]=get_dp_den(in->xp[i][band]-in->tp[i][band],in->Th[i],in->mh[i],in->dostype[i],dos_all);



			}


			//ntc=get_n_trap_den(in->xt[i]+in->tt[i],in->Te[i],in->me[i],in->dostype[i],0);
			//dntc=get_dn_trap_den(in->xt[i]+in->tt[i],in->Te[i],in->me[i],in->dostype[i],0);

			//ptc=get_p_trap_den(in->xpt[i]-in->tpt[i],in->Th[i],in->mh[i],in->dostype[i],0);
			//dptc=get_dp_trap_den(in->xpt[i]-in->tpt[i],in->Th[i],in->mh[i],in->dostype[i],0);



			if (in->ntrapnewton)
			{
				ntc=get_n_trap_den(in->xt[i]+in->tt[i],in->Te[i],in->me[i],in->dostype[i],0);
				dntc=get_dn_trap_den(in->xt[i]+in->tt[i],in->Te[i],in->me[i],in->dostype[i],0);
			}else if (in->combitrap)
			{
				ntc=get_n_trap_den(in->x[i][0]+in->t[i][0],in->Te[i],in->me[i],in->dostype[i],0);
				dntc=get_dn_trap_den(in->x[i][0]+in->t[i][0],in->Te[i],in->me[i],in->dostype[i],0);
			}else
			{
				ntc=0.0;
				dntc=0.0;
			}

			//printf("%e %e\n",nc[0],ntc);
			//getchar();

			if (in->ptrapnewton)
			{
				ptc=get_p_trap_den(in->xpt[i]-in->tpt[i],in->Th[i],in->mh[i],in->dostype[i],0);
				dptc=get_dp_trap_den(in->xpt[i]-in->tpt[i],in->Th[i],in->mh[i],in->dostype[i],0);
			}else if (in->combitrap)
			{
				ptc=get_p_trap_den(in->xp[i][0]-in->tp[i][0],in->Th[i],in->mh[i],in->dostype[i],0);
				dptc=get_dp_trap_den(in->xp[i][0]-in->tp[i][0],in->Th[i],in->mh[i],in->dostype[i],0);
			}
			else
			{
				ptc=0.0;
				dptc=0.0;
			}



			klc=in->kl[i];
			nlast=in->nlast[i];
			plast=in->plast[i];
			dt=in->dt;

	R=in->R[i];
	Gex=in->Gex[i];

	e0=(epl+epc)/2.0;
	e1=(epc+epr)/2.0;

	kl0=(klc+kll)/2.0;
	kl1=(klr+klc)/2.0;

	dphil=-e0/dyl/ddh;
	dphic=e0/dyl/ddh+e1/dyr/ddh;
	dphir=-e1/dyr/ddh;
	Rbi=0.0;
	deriv=phil*dphil+phic*dphic+phir*dphir;

	for (band=0;band<bands;band++)
	{
		dphidxic[band]=q*(dnc[band]);
		dphidxipc[band]=-q*(dpc[band]);

		if (in->ntrapnewton==TRUE)
		{
			dphidxic[band]+=q*(in->dnt[i]);
			dphidntrap=q*(dntc);
		}

		if (in->ptrapnewton==TRUE)
		{
			dphidptrap=-q*(dptc);
			dphidxipc[band]+=-q*(in->dpt[i]);
		}

		//if (in->combitrap)
		//{
		//	dphidxic[band]+=q*(in->dnt[i]);
		//	dphidxipc[band]+=-q*(in->dpt[i]);
		//}
	}



	//double kdiss=0.0;
	double mu_min=0.0;

	double langevin=0.0;

	if (in->Rbitype==0)
	{
		mu_min=in->mun_avg[i];
		if (mu_min>in->mup_avg[i]) mu_min=in->mup_avg[i];
		langevin=q*(mu_min)/(epc);
		Rbi_mul=1.0;

	}else
	if (in->Rbitype==1)
	{
		mu_min=(in->alpha*in->mun_avg[i]+in->beta*in->mup_avg[i])/2.0;
		langevin=q*(mu_min)/(epc);
		//printf("%e\n",langevin);
		Rbi_mul=1.0;

	}else
	if (in->Rbitype==2)
	{
		mu_min=(in->mup_avg[i]+in->mun_avg[i])/2.0;
		langevin=q*(mu_min)/(epc);
		//printf("%e\n",langevin);
		double rc=q*q/(4.0*pi*epsilon0*in->epsilonr[i]*kb*Tlc);
		double A=in->A[i];
		double deltaE=in->deltaE[i];
		//printf("%e %e\n",deltaE,A);
		double R=in->a[i];
		double Dp=A*exp(-(deltaE*q/(kb*Tlc)));
		Rbi_mul=1.0/(1-exp(-rc/R)+(Dp*rc/R/R)*exp(-rc/R));
		//printf("%e %e\n",q*(mu_min)*exp(-q*0.178/(kb*Tlc))/(epc),Rbi_mul*langevin);
	}
	else
	if (in->Rbitype==3)
	{
		Rbi_mul=1.0;
		mu_min=(in->mup_avg[i]+in->mun_avg[i])/2.0;
		langevin=q*(mu_min)*exp(-q*in->deltaE[i]/(kb*Tlc))/(epc);
	}else
	if (in->Rbitype==4)
	{
		Rbi_mul=1.0;
		langevin=get_mat_param_xy(&(in->mat.l[in->ymatlayers[i]]),mat_rbi,sqrt(fabs(nc[0]*pc[0]-ni*ni)));
		//printf("Here I am %e\n",langevin);
		if (langevin<0) langevin=1e-30;
	}else
	if (in->Rbitype==5)
	{
		//printf("Calculating with this\n");
		Rbi_mul=deibel;
		//printf("%e\n",deibel);
		langevin=(in->mup_avg[i]+in->mun_avg[i])/2.0;
	}else
	if (in->Rbitype==6)
	{
		//printf("Calculating with this\n");
		Rbi_mul=1.0;
		//printf("%e\n",deibel);
		langevin=q*2.0*sqrt(in->mup_avg[i]*in->mun_avg[i])/epc;
	}
	//Rbi_mul=0.0;
	//printf("%e %e %e\n",langevin,nc*pc-ni*ni,sqrt(fabs(nc*pc-ni*ni)));
	in->Rbi_k[i]=Rbi_mul*langevin;
	Rbi=Rbi_mul*langevin*(nc[0]*pc[0]-in->ninit[i]*in->pinit[i]);//

	//printf ("%e\n",in->ninit[i]);
	if (in->Rbi_on==FALSE)
	{
		Rbi=0.0;
		Rbi_mul=0.0;
	}

	if ((in->Vapplied==0.0)&&(in->Psun==0.0))
	{
		Rbi=0.0;
		Rbi_mul=0.0;
	}

	in->Rbi[i]=Rbi;


	double dRbidxic=Rbi_mul*langevin*(dnc[0]*pc[0]);
	double dRbidxipc=Rbi_mul*langevin*(nc[0]*dpc[0]);

	G=in->G[i];
//printf("%e %e\n",R,G);
			for (band=0;band<bands;band++)
			{

xil[band]=q*2.0*(3.0/2.0)*(Ecc-Ecl)/((wnc[band]+wnl[band]))-2.0*(3.0/2.0)*(mec-mel)/(mec+mel)+2.0*(wnc[band]-wnl[band])/(wnc[band]+wnl[band]);
xir[band]=q*2.0*(3.0/2.0)*(Ecr-Ecc)/((wnr[band]+wnc[band]))-2.0*(3.0/2.0)*(mer-mec)/(mer+mec)+2.0*(wnr[band]-wnc[band])/(wnr[band]+wnc[band]);

xipl[band]=q*2.0*(3.0/2.0)*(Evc-Evl)/((wpc[band]+wpl[band]))-2.0*(3.0/2.0)*(mhc-mhl)/(mhc+mhl)-2.0*(wpc[band]-wpl[band])/(wpc[band]+wpl[band]);
xipr[band]=q*2.0*(3.0/2.0)*(Evr-Evc)/((wpr[band]+wpc[band]))-2.0*(3.0/2.0)*(mhr-mhc)/(mhr+mhc)-2.0*(wpr[band]-wpc[band])/(wpr[band]+wpc[band]);

//printf("%e %e %e\n",q*2.0*(3.0/2.0)*(Ecc-Ecl)/((wnc[band]+wnl[band])),2.0*(3.0/2.0)*(mec-mel)/(mec+mel),2.0*(wnc[band]-wnl[band])/(wnc[band]+wnl[band]));
//getchar();

			Jnl[band]=(Dnl[band]/dyl)*(B(-xil[band])*nc[band]-B(xil[band])*nl[band]);
			Jnr[band]=(Dnr[band]/dyr)*(B(-xir[band])*nr[band]-B(xir[band])*nc[band]);

			Jpl[band]=(Dpl[band]/dyl)*(B(-xipl[band])*pl[band]-B(xipl[band])*pc[band]);
			Jpr[band]=(Dpr[band]/dyr)*(B(-xipr[band])*pc[band]-B(xipr[band])*pr[band]);

			if (i==0)
			{
				in->Jnleft=Jnl[band];
				in->Jpleft=Jpl[band];
			}

			if (i==in->ymeshpoints-1)
			{
				in->Jnright=Jnr[band];
				in->Jpright=Jpr[band];
			}
//fprintf(file_j,"%e %e\n",in->ymesh[i],-(get_top_from_p(pc[band],in->Te[0],in->mh[0],in->Nv[0],in->dostype[i],dos_all)+in->Xi[i]+in->Eg[i]+in->phi[i]));
//fprintf(file_j,"%e %e\n",in->ymesh[i]-dyl/2.0,Jnl[band]);
//fprintf(file_j,"%e %e\n",in->ymesh[i]+dyl/2.0,Jnr[band]);
//fprintf(file_j,"%e %e %e\n",in->ymesh[i]-dyl/2.0,Jnl[band],Jpl[band]);
//fprintf(file_j,"%e %e %e\n",in->ymesh[i]+dyr/2.0,Jnr[band],Jpr[band]);

			in->Jn[i][band]=q*(Jnl[band]+Jnr[band])/2.0;
			in->Jp[i][band]=q*(Jpl[band]+Jpr[band])/2.0;
			//double jntest=munc[0]*nc[0]*(Ecr-Ecl)/ddh+kb*in->Te[i]*munc[0]*(nr[0]-nl[0])/ddh;
//fprintf(file_j,"%e %e %e\n",in->ymesh[i]-dyl/2.0,Jnl[band],jntest);
//fprintf(file_j,"%e %e\n",in->ymesh[i]+dyl/2.0,Jnr[band]);
			dJdxil[band]=(Dnl[band]/dyl/ddh)*(B(xil[band])*dnl[band]);
			dJdxic[band]=-(Dnl[band]/dyl)*B(-xil[band])*dnc[band]/ddh-(Dnr[band]/dyr)*B(xir[band])*dnc[band]/ddh;
			dJdxir[band]=(Dnr[band]/dyr)*B(-xir[band])*dnr[band]/ddh;


			dJpdxipl[band]=-(Dpl[band]/dyl/ddh)*(B(-xipl[band])*dpl[band]);
			dJpdxipc[band]=((Dpl[band]/dyl/ddh)*B(xipl[band])*dpc[band]+(Dpr[band]/dyr/ddh)*B(-xipr[band])*dpc[band]);
			dJpdxipr[band]=-(Dpr[band]/dyr/ddh)*B(xipr[band])*dpr[band];

			dJdphil[band]=(munl[band]/dyl/ddh)*(dB(-xil[band])*nc[band]+dB(xil[band])*nl[band]);
			dJdphic[band]=-((munl[band]/dyl)*(dB(-xil[band])*nc[band]+dB(xil[band])*nl[band])/ddh+(munr[band]/dyr)*(dB(-xir[band])*nr[band]+dB(xir[band])*nc[band])/ddh);
			dJdphir[band]=(munr[band]/dyr/ddh)*(dB(-xir[band])*nr[band]+dB(xir[band])*nc[band]);


			dJpdphil[band]=((mupl[band])/dyl/ddh)*(dB(-xipl[band])*pl[band]+dB(xipl[band])*pc[band]);
			dJpdphic[band]=-(((mupl[band])/dyl/ddh)*(dB(-xipl[band])*pl[band]+dB(xipl[band])*pc[band])+(mupr[band]/dyr/ddh)*(dB(-xipr[band])*pc[band]+dB(xipr[band])*pr[band]));
			dJpdphir[band]=(mupr[band]/dyr/ddh)*(dB(-xipr[band])*pc[band]+dB(xipr[band])*pr[band]);


			}


			for (band=0;band<bands;band++)
			{
				if (in->go_time==TRUE)
				{
					dJdxic[band]+=-dnc[band]/dt;
				}

				if (in->go_time==TRUE)
				{
					dJpdxipc[band]+=dpc[band]/dt;
				}

			}

			dJdxipc=0.0;
			if (in->Rbi_on==TRUE)
			{
				dJdxic[0]+=-dRbidxic;
				dJdxipc+=-dRbidxipc;
			}




			dJpdxic=0.0;
			if (in->Rbi_on==TRUE)
			{
				dJpdxipc[0]+=dRbidxipc;
				dJpdxic+=dRbidxic;
			}




			if (in->ntrapnewton==TRUE)
			{
				dntrapdntrap=-dntc/taunescapec;
				dntrapdn=dnc[0]/tauncapc;
				dntrap=nc[0]/tauncapc-ntc/taunescapec;
			}

			if (in->ptrapnewton==TRUE)
			{
				dptrapdptrap=-dptc/taupescapec;
				dptrapdp=dpc[0]/taupcapc;
				dptrap=pc[0]/taupcapc-ptc/taupescapec;
			}

			if (i!=0)
			{
				in->Ti[pos]=i;
				in->Tj[pos]=i-1;
				in->Tx[pos]=dphil;
				pos++;
				//electron
				for (band=0;band<bands;band++)
				{
					in->Ti[pos]=in->ymeshpoints*(1+band)+i;
					in->Tj[pos]=in->ymeshpoints*(1+band)+i-1;
					in->Tx[pos]=dJdxil[band];
					pos++;

					in->Ti[pos]=in->ymeshpoints*(1+band)+i;
					in->Tj[pos]=i-1;
					in->Tx[pos]=dJdphil[band];
					pos++;

					//hole
					in->Ti[pos]=in->ymeshpoints*(1+bands+band)+i;
					in->Tj[pos]=in->ymeshpoints*(1+bands+band)+i-1;
					in->Tx[pos]=dJpdxipl[band];
					pos++;

					in->Ti[pos]=i+in->ymeshpoints*(1+bands+band);
					in->Tj[pos]=i-1;
					in->Tx[pos]=dJpdphil[band];
					pos++;

				}


			}

			//phi
			in->Ti[pos]=i;
			in->Tj[pos]=i;
			in->Tx[pos]=dphic;
			pos++;

			for (band=0;band<bands;band++)
			{
			in->Ti[pos]=i;
			in->Tj[pos]=i+in->ymeshpoints*(1+band);
			in->Tx[pos]=dphidxic[band];
			pos++;

			in->Ti[pos]=i;
			in->Tj[pos]=i+in->ymeshpoints*(1+bands+band);
			in->Tx[pos]=dphidxipc[band];
			pos++;
			}

			//electron
			for (band=0;band<bands;band++)
			{
			in->Ti[pos]=in->ymeshpoints*(1+band)+i;
			in->Tj[pos]=in->ymeshpoints*(1+band)+i;
			in->Tx[pos]=dJdxic[band];
			pos++;


			in->Ti[pos]=in->ymeshpoints*(1+band)+i;
			in->Tj[pos]=i;
			in->Tx[pos]=dJdphic[band];
			pos++;


			}


			//hole
			for (band=0;band<bands;band++)
			{
			in->Ti[pos]=in->ymeshpoints*(1+bands+band)+i;
			in->Tj[pos]=in->ymeshpoints*(1+bands+band)+i;
			in->Tx[pos]=dJpdxipc[band];
			pos++;


			in->Ti[pos]=in->ymeshpoints*(1+bands+band)+i;
			in->Tj[pos]=i;
			in->Tx[pos]=dJpdphic[band];
			pos++;

			}


			if (in->ntrapnewton==TRUE)
			{
				in->Ti[pos]=in->ymeshpoints*(1+bands+bands)+i;
				in->Tj[pos]=in->ymeshpoints*(1+bands+bands)+i;
				in->Tx[pos]=dntrapdntrap;
				pos++;

				in->Ti[pos]=in->ymeshpoints*(1+bands+bands)+i;
				in->Tj[pos]=in->ymeshpoints*1+i;
				in->Tx[pos]=dntrapdn;
				pos++;

				in->Ti[pos]=i;
				in->Tj[pos]=in->ymeshpoints*(1+bands+bands)+i;
				in->Tx[pos]=dphidntrap;
				pos++;


			}

			if (in->ptrapnewton==TRUE)
			{
				in->Ti[pos]=in->ymeshpoints*(1+bands+bands+1)+i;
				in->Tj[pos]=in->ymeshpoints*(1+bands+bands+1)+i;
				in->Tx[pos]=dptrapdptrap;
				pos++;

				in->Ti[pos]=in->ymeshpoints*(1+bands+bands+1)+i;
				in->Tj[pos]=in->ymeshpoints*(1+bands)+i;
				in->Tx[pos]=dptrapdp;
				pos++;

				in->Ti[pos]=i;
				in->Tj[pos]=in->ymeshpoints*(1+bands+bands+1)+i;
				in->Tx[pos]=dphidptrap;
				pos++;


			}

			if (i!=(in->ymeshpoints-1))
			{
				//phi
				in->Ti[pos]=i;
				in->Tj[pos]=i+1;
				in->Tx[pos]=dphir;
				pos++;


				for (band=0;band<bands;band++)
				{

				//electron
				in->Ti[pos]=in->ymeshpoints*(1+band)+i;
				in->Tj[pos]=in->ymeshpoints*(1+band)+i+1;
				in->Tx[pos]=dJdxir[band];
				pos++;

				in->Ti[pos]=i+in->ymeshpoints*(1+band);
				in->Tj[pos]=i+1;
				in->Tx[pos]=dJdphir[band];
				pos++;

				//hole
				in->Ti[pos]=in->ymeshpoints*(1+bands+band)+i;
				in->Tj[pos]=in->ymeshpoints*(1+bands+band)+i+1;
				in->Tx[pos]=dJpdxipr[band];
				pos++;

				in->Ti[pos]=i+in->ymeshpoints*(1+bands+band);
				in->Tj[pos]=i+1;
				in->Tx[pos]=dJpdphir[band];
				pos++;
				}




			}

			//Possion
			in->b[i]=-(deriv);
			for (band=0;band<bands;band++)
			{
				in->b[i]+=-(-q*(in->p[i][band]-in->n[i][band]));
			}

			for (band=0;band<bands;band++)
			{
			//electron
			in->b[in->ymeshpoints*(1+band)+i]=-((Jnr[band]-Jnl[band])/ddh-Rbi);
			//printf("%e %e\n",Rbi,in->Rbi[i]);
			if (in->go_time==TRUE)
			{
				in->b[in->ymeshpoints*(1+band)+i]-=-(nc[0]-nlast)/dt;
			//	printf("%e %e\n",nc[0],nlast);
			}
			//getchar();

			if (in->Dphoton==TRUE) in->b[in->ymeshpoints*(1+band)+i]-=in->Gex[i]*in->Dphotoneff;
//printf("%e %e\n",Rbi,in->Gex[i]*in->Dphotoneff);
			//hole
			in->b[in->ymeshpoints*(1+bands+band)+i]=-((Jpr[band]-Jpl[band])/ddh+Rbi);

			if (in->Dphoton==TRUE) in->b[in->ymeshpoints*(1+bands+band)+i]-=-in->Gex[i]*in->Dphotoneff;
			if (in->go_time==TRUE) in->b[in->ymeshpoints*(1+bands+band)+i]-=(pc[0]-plast)/dt;
			}

			if (in->ntrapnewton==TRUE)
			{
				in->b[in->ymeshpoints*(1+bands+bands)+i]=-(dntrap);
			}

			if (in->ptrapnewton==TRUE)
			{
				in->b[in->ymeshpoints*(1+bands+bands+1)+i]=-(dptrap);
			}

		if (pos>in->N)
		{
			printf("Error\n");
		}
		}



//	solver_dump_matrix(in->M,pos,in->Ti,in->Tj, in->Tx,in->b);
//fclose(file_j);
//printf("Check J file\n");
//getchar();

}

double get_cur_error(struct device *in)
{
int i;
double phi=0.0;
double n=0.0;
double p=0.0;
double x=0.0;
double te=0.0;
double th=0.0;
double tl=0.0;
double ttn=0.0;
double ttp=0.0;
int band;
int bands=in->bands;
for (i=0;i<in->ymeshpoints;i++)
{

#ifdef only
	if (only_update_thermal==FALSE)
	{
#endif
		phi+=mod(in->b[i]);
		for (band=0;band<in->bands;band++)
		{
			n+=mod(in->b[in->ymeshpoints*(1+band)+i]);
			p+=+mod(in->b[in->ymeshpoints*(1+bands+band)+i]);
		}

		if (in->ntrapnewton==TRUE)
		{
			ttn+=mod(in->b[in->ymeshpoints*(1+bands+bands)+i]);
		}

		if (in->ptrapnewton==TRUE)
		{
			ttp+=mod(in->b[in->ymeshpoints*(1+bands+bands+1)+i]);
		}

		if (in->enable_ex==TRUE)
		{
			x+=mod(in->b[3*in->ymeshpoints+i])/1e32;
		}

		if (in->nnewton==TRUE)
		{
			te+=mod(in->b[4*in->ymeshpoints+i]);
		}

#ifdef only

		}else
		{
#endif
		if (in->hnewton==TRUE)
		{
			th+=mod(in->b[5*in->ymeshpoints+i]);
		}

		if (in->lnewton==TRUE)
		{
			tl+=mod(in->b[6*in->ymeshpoints+i]);
		}


#ifdef only
	}
#endif
}

double tot=phi+n+p+x+te+th+tl+ttn+ttp;
//printf("%e %e %e %e %e %e %e %e %e\n",phi,n,p,x,te,th,tl,ttn,ttp);
if (isnan( tot))
{
	printf("%e %e %e %e %e %e %e %e %e\n",phi,n,p,x,te,th,tl,ttn,ttp);
	solver_dump_matrix(in->M,in->N,in->Ti,in->Tj, in->Tx,in->b);
	exit(0);
}

return tot;
}

void solver_get_memory(struct device *in)
{
int i;
int bands=in->bands;
in->N=in->ymeshpoints*3-2;	//Possion main

for (i=0;i<bands;i++)
{
	in->N+=in->ymeshpoints*3-2;	//Je main
	in->N+=in->ymeshpoints*3-2;	//Jh main
}

if (in->enable_ex==TRUE)
{
in->N+=in->ymeshpoints*3-2;	//ex main
in->N+=in->ymeshpoints;		//ex dn
in->N+=in->ymeshpoints;		//ex dp
in->N+=in->ymeshpoints;		//dJndex
in->N+=in->ymeshpoints;		//dJpdex

}

for (i=0;i<in->bands;i++)
{
	in->N+=in->ymeshpoints*3-2;	//dJe/phi
	in->N+=in->ymeshpoints*3-2;	//dJh/phi
}

for (i=0;i<in->bands;i++)
{
	in->N+=in->ymeshpoints;		//dphi/dn
	in->N+=in->ymeshpoints;		//dphi/dh
}

//in->N+=in->ymeshpoints;		//dJndp
//in->N+=in->ymeshpoints;		//dJpdn

in->M=in->ymeshpoints;	//Pos

for (i=0;i<in->bands;i++)
{
	in->M+=in->ymeshpoints;		//Je
	in->M+=in->ymeshpoints;		//Jh
}

if (in->enable_ex==TRUE)
{
in->M+=in->ymeshpoints;		//ex
}

if (in->ntrapnewton==TRUE)
{
in->N+=in->ymeshpoints;		//dntrapdn
in->N+=in->ymeshpoints;		//dntrapdntrap
in->N+=in->ymeshpoints;		//dphidntrap

in->M+=in->ymeshpoints;		//nt
}

if (in->ptrapnewton==TRUE)
{
in->N+=in->ymeshpoints;		//dptrapdp
in->N+=in->ymeshpoints;		//dptrapdptrap
in->N+=in->ymeshpoints;		//dphidptrap

in->M+=in->ymeshpoints;		//pt
}


if (in->nnewton==TRUE)
{
in->N+=in->ymeshpoints*3-2;	//dTedTe
in->N+=in->ymeshpoints*3-2;	//dTedphi
in->N+=in->ymeshpoints*3-2;	//dJedTe
in->N+=in->ymeshpoints;		//dphidTe
in->N+=in->ymeshpoints*3-2;	//dTedn

in->M+=in->ymeshpoints;		//Te

}

/*if (in->nnewton==TRUE)
{
in->N+=in->ymeshpoints*3-2;	//dTedTe
in->N+=in->ymeshpoints*3-2;	//dTedphi
in->N+=in->ymeshpoints*3-2;	//dJedTe
in->N+=in->ymeshpoints;		//dphidTe
in->N+=in->ymeshpoints*3-2;	//dTedn

in->M+=in->ymeshpoints;		//Te

}*/

if (in->hnewton==TRUE)
{
in->N+=in->ymeshpoints*3-2;	//dThdTh
in->N+=in->ymeshpoints*3-2;		//dThdphi
in->N+=in->ymeshpoints*3-2;	//dThdTp
in->N+=in->ymeshpoints*3-2;	//dJpdTh
in->M+=in->ymeshpoints;		//Th
}

if (in->lnewton==TRUE)
{
in->N+=in->ymeshpoints*3-2;	//dTldTl
in->N+=in->ymeshpoints;		//dTldTe
in->N+=in->ymeshpoints;		//dTldTh
in->N+=in->ymeshpoints;		//dTldnc
in->N+=in->ymeshpoints;		//dTldcp
in->M+=in->ymeshpoints;		//Tl
}

in->Ti =    malloc(in->N*sizeof(int));
in->Tj =    malloc(in->N*sizeof(int));
in->Tx = malloc(in->N*sizeof(double));
in->b = malloc(in->M*sizeof(double));
in->back_b = malloc(in->M*sizeof(double));

Jnl=(double*)malloc(bands*sizeof(double));
Jnr=(double*)malloc(bands*sizeof(double));
Jpl=(double*)malloc(bands*sizeof(double));
Jpr=(double*)malloc(bands*sizeof(double));

Dnl=(double*)malloc(bands*sizeof(double));
Dnc=(double*)malloc(bands*sizeof(double));
Dnr=(double*)malloc(bands*sizeof(double));
Dpl=(double*)malloc(bands*sizeof(double));
Dpc=(double*)malloc(bands*sizeof(double));
Dpr=(double*)malloc(bands*sizeof(double));

nl=(double*)malloc(bands*sizeof(double));
nc=(double*)malloc(bands*sizeof(double));
nr=(double*)malloc(bands*sizeof(double));

pl=(double*)malloc(bands*sizeof(double));
pc=(double*)malloc(bands*sizeof(double));
pr=(double*)malloc(bands*sizeof(double));

xil=(double*)malloc(bands*sizeof(double));
xir=(double*)malloc(bands*sizeof(double));
xipl=(double*)malloc(bands*sizeof(double));
xipr=(double*)malloc(bands*sizeof(double));

dJpdxipl=(double*)malloc(bands*sizeof(double));
dJpdxipc=(double*)malloc(bands*sizeof(double));
dJpdxipr=(double*)malloc(bands*sizeof(double));

dnl=(double*)malloc(bands*sizeof(double));
dnc=(double*)malloc(bands*sizeof(double));
dnr=(double*)malloc(bands*sizeof(double));

dpl=(double*)malloc(bands*sizeof(double));
dpc=(double*)malloc(bands*sizeof(double));
dpr=(double*)malloc(bands*sizeof(double));

munl=(double*)malloc(bands*sizeof(double));
munc=(double*)malloc(bands*sizeof(double));
munr=(double*)malloc(bands*sizeof(double));

mupl=(double*)malloc(bands*sizeof(double));
mupc=(double*)malloc(bands*sizeof(double));
mupr=(double*)malloc(bands*sizeof(double));


wnl=(double*)malloc(bands*sizeof(double));
wnc=(double*)malloc(bands*sizeof(double));
wnr=(double*)malloc(bands*sizeof(double));

wpl=(double*)malloc(bands*sizeof(double));
wpc=(double*)malloc(bands*sizeof(double));
wpr=(double*)malloc(bands*sizeof(double));

dnldTe=(double*)malloc(bands*sizeof(double));
dncdTe=(double*)malloc(bands*sizeof(double));
dnrdTe=(double*)malloc(bands*sizeof(double));

dpldTh=(double*)malloc(bands*sizeof(double));
dpcdTh=(double*)malloc(bands*sizeof(double));
dprdTh=(double*)malloc(bands*sizeof(double));

nc_all=(double*)malloc(bands*sizeof(double));
pc_all=(double*)malloc(bands*sizeof(double));
dnc_all=(double*)malloc(bands*sizeof(double));
dpc_all=(double*)malloc(bands*sizeof(double));


dJdxil=(double*)malloc(bands*sizeof(double));
dJdxic=(double*)malloc(bands*sizeof(double));
dJdxir=(double*)malloc(bands*sizeof(double));

dJdphil=(double*)malloc(bands*sizeof(double));
dJdphic=(double*)malloc(bands*sizeof(double));
dJdphir=(double*)malloc(bands*sizeof(double));

dJpdphil=(double*)malloc(bands*sizeof(double));
dJpdphic=(double*)malloc(bands*sizeof(double));
dJpdphir=(double*)malloc(bands*sizeof(double));


dphidxic=(double*)malloc(bands*sizeof(double));
dphidxipc=(double*)malloc(bands*sizeof(double));
}

void solver_free_memory(struct device *in)
{
free(in->Ti);
free(in->Tj);
free(in->Tx);
free(in->b);
free(in->back_b);


free(Jnl);
free(Jnr);
free(Jpl);
free(Jpr);

free(Dnl);
free(Dnc);
free(Dnr);
free(Dpl);
free(Dpc);
free(Dpr);

free(nl);
free(nc);
free(nr);

free(pl);
free(pc);
free(pr);

free(xil);
free(xir);
free(xipl);
free(xipr);

free(dJpdxipl);
free(dJpdxipc);
free(dJpdxipr);

free(dnl);
free(dnc);
free(dnr);

free(dpl);
free(dpc);
free(dpr);

free(munl);
free(munc);
free(munr);

free(mupl);
free(mupc);
free(mupr);


free(wnl);
free(wnc);
free(wnr);

free(wpl);
free(wpc);
free(wpr);

free(dnldTe);
free(dncdTe);
free(dnrdTe);


free(dpldTh);
free(dpcdTh);
free(dprdTh);

free(nc_all);
free(pc_all);
free(dnc_all);
free(dpc_all);


free(dJdxil);
free(dJdxic);
free(dJdxir);

free(dJdphil);
free(dJdphic);
free(dJdphir);


free(dJpdphil);
free(dJpdphic);
free(dJpdphir);

free(dphidxic);
free(dphidxipc);
}

void dump_for_plot(struct device *in)
{

int band;
int bands=in->bands;
if (in->enable_plot)
{
	int i;
	FILE *out =fopen ("./band.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
			fprintf(out,"%e %e %e %e %e %e %e %e\n",in->ymesh[i],in->Ec[i],in->Ev[i],in->Fi[i],in->Fn[i][0],in->Fp[i][0],in->Fnt[i],in->Fpt[i]);
	}

	fclose(out);

	out =fopen ("./phi.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
			fprintf(out,"%e %e\n",in->ymesh[i],in->phi[i]);
	}

	fclose(out);

	out =fopen ("./j.dat","w");
	double Jtot=0.0;
	for (i=0;i<in->ymeshpoints;i++)
	{
		fprintf(out,"%e ",in->ymesh[i]);
		Jtot=0.0;

		for (band=0;band<bands;band++)
		{
			fprintf(out,"%e %e ",in->Jn[i][band],in->Jp[i][band]);
			Jtot+=in->Jn[i][band]+in->Jp[i][band];
		}

		fprintf(out,"%e \n",Jtot);

	}

	fclose(out);

	out =fopen ("./Fn.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
		fprintf(out,"%e ",in->ymesh[i]);

		for (band=0;band<bands;band++)
		{
			fprintf(out,"%e ",in->Fn[i][band]);
		}

		fprintf(out,"\n");

	}

	fclose(out);

	out =fopen ("./Fp.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
		fprintf(out,"%e ",in->ymesh[i]);

		for (band=0;band<bands;band++)
		{
			fprintf(out,"%e ",in->Fp[i][band]);
		}

		fprintf(out,"\n");

	}

	fclose(out);


	out =fopen ("./mu.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
		fprintf(out,"%e ",in->ymesh[i]);

		for (band=0;band<bands;band++)
		{
			fprintf(out,"%e %e ",in->mun[i][band],in->mup[i][band]);
		}

		fprintf(out,"\n");

	}

	fclose(out);

	out =fopen ("./n.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
		fprintf(out,"%e ",in->ymesh[i]);

		for (band=0;band<bands;band++)
		{
			fprintf(out,"%e %e ",in->n[i][band]-in->nt[i],in->p[i][band]-in->pt[i]);
		}

		fprintf(out,"\n");
	}

	fclose(out);


	out =fopen ("./trap.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
		fprintf(out,"%e ",in->ymesh[i]);

		fprintf(out,"%e %e ",in->nt[i],in->pt[i]);

		fprintf(out,"\n");
	}

	fclose(out);


	out =fopen ("./Dex.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
			fprintf(out,"%e %e %e %e\n",in->ymesh[i],in->ex[i],in->G[i],in->Gex[i]);
	}

	fclose(out);

	out =fopen ("./R.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
			fprintf(out,"%e %e\n",in->ymesh[i],in->Rbi[i]);
	}

	if ((in->nnewton==TRUE)||(in->hnewton==TRUE))
	{
		dump_thermal(in);
	}
	fclose(out);

}
}

int solve_cur_thermal(struct device *in,int thermal)
{
//printf("I am here %d",thermal);
//getchar();
int ittr=0;
int cont=TRUE;



if (thermal==FALSE)
{

	solve_cur(in);
}else
{
		do
		{
			solve_cur(in);
			solve_thermal(in);
			//printf("ii=%d\n",ii);
			plot_now(in,"plot");
			///getchar();
		if (((in->thermal_conv==TRUE)&&(in->dd_conv==TRUE))||(ittr>10)) cont=FALSE;
		//printf("%d %d %d\n",in->thermal_conv,in->dd_conv,ii);
		//getchar();
		ittr++;
		}while(cont==TRUE);

}

return 0;
}

int solve_cur(struct device *in)
{
double error;
int ittr=0;
dprintf("Solve cur\n");




int doback=FALSE;
#ifdef only
only_update_thermal=FALSE;
#endif
//in->enable_back=FALSE;
int keep_going=FALSE;
int thermalrun=0;
	do
	{

		fill_matrix(in);
//dump_for_plot(in);
//plot_now(in,"plot");
	//solver_dump_matrix(in->M,in->N,in->Ti,in->Tj, in->Tx,in->b);
//getchar();
		if (in->enable_back)
		{
			if (ittr<5)
			{
				doback=TRUE;
			}else
			{
				doback=FALSE;
			}
		}
		//doback=FALSE;
		//if (ittr==6) doback=TRUE;

		//if (error<1e-6) doback=TRUE;

		doback=FALSE;
		//if (ittr<3) doback=FALSE;
		//doback=TRUE;
		//in->enable_back=FALSE;
		if (in->stop==TRUE)
		{
			break;
		}



		if (doback==TRUE)
		{
			do_backtrack(in);
			update_solver_vars(in,FALSE);
		}else
		{
			solver(in->M,in->N,in->Ti,in->Tj, in->Tx,in->b);
			update_solver_vars(in,TRUE);
			//getchar();
		}


		//thermalrun++;
		if (thermalrun==40) thermalrun=0;
		//update(in);
//getchar();
		error=get_cur_error(in);
		dprintf("%d Cur error = %e %e",ittr,error,in->Vapplied);
		in->last_error=error;
		if (doback==TRUE)
		{
			dprintf(" b");
		}


		dprintf("\n");
		ittr++;

		#ifdef dumpconverge
		in->converge=fopen("./converge.dat","a");
		fprintf(in->converge,"%e\n",error);
		fclose(in->converge);
		#endif
		keep_going=FALSE;

		quick_update();

		//printf("%le\n",in->min_cur_error);
		//getchar();
		in->dd_conv=FALSE;
		if ((ittr<2)&&(error<in->min_cur_error)) in->dd_conv=TRUE;
		if ((ittr<in->max_electrical_itt)&&(error>in->min_cur_error)) keep_going=TRUE;

	}while(keep_going==TRUE);
//getchar();
		//int r;
		//for (r=0;r<in->ymeshpoints;r++)
		//{
		//in->mun[r]=(get_mat_param_mobility(&(in->mat.l[in->ymatlayers[r]]),mat_mue,in->n[r])-in->mun[r])/100.0;
		//in->mup[r]=(get_mat_param_mobility(&(in->mat.l[in->ymatlayers[r]]),mat_muh,in->p[r])-in->mup[r])/100.0;
		//}
dump_for_plot(in);
plot_now(in,"plot");
//getchar();
in->odes+=in->M;
//getchar();
return 0;
}

